/*
 *  Filename:  ted.c
 *
 *  Version 1.3a
 *
 *  Copyright 1994 by Br. David Carlson
 *
 *  Programmer:  Br. David Carlson
 *
 *  Date:  July 23, 1994
 *
 *  Last modified:  August 4, 1995
 *
 *  Use ted <filename> for a simple way to edit a file.  Once started,
 *  use F1 for help.   You can also use ted <filename> -n to specify the
 *  maximum number of pages n that you can edit.  Note that n must be at
 *  least 1 and no more than 100.  Pages are considered to be 50 lines long.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "screen.h"
#include "ted.h"


/* Global variables for quick access to commonly used items: */

LINE_TYPE *buffer;
MARK_RECORD mark;
char source_filename[80], TargetString[32];
INT8 mode, proceed, insert, modified, BlockMarked;
INT16 line_count, avail, first, last, current, file_row, clipboard,
   screen_row, screen_col, file_col, TargetLength;
INT32 buffer_size;


/*  The following function gives a "syntax error" error message and exits. */

void syntax_error(void)
   {
   fprintf(stderr, "Error in syntax.  Use:\n");
   fprintf(stderr, "ted <filename>\n");
   fprintf(stderr, "   or\n");
   fprintf(stderr, "ted <filename> -n   (where 1 <= n <= 100)\n");
   exit(1);
   }


/* The following function does all of the processing when a printable
 * key has been pressed.
 */

void process_printable(int key)
   {
   register int k;
   INT16 len;
   LINE_TYPE *buf_ptr;

   buf_ptr = &buffer[current];
   len = buf_ptr->length;
   if (insert)
      {
      if (len == 80)
         beep();
      else if (screen_col >= len)   /* there is nothing to move over */
         {
         putchar(key);
         modified = TRUE;
         buf_ptr->info[screen_col] = key;
         if (screen_col == 79)
            {
            buf_ptr->length = 80;
            beep();
            }
         else
            {
            screen_col++;
            file_col++;
            buf_ptr->length = screen_col;
            }
         }
      else
         {
         for (k = len; k > screen_col;  k--)
            buf_ptr->info[k] = buf_ptr->info[k - 1];
         buf_ptr->info[screen_col] = key;
         for (k = screen_col; k <= len; k++)
            putchar(buf_ptr->info[k]);
         screen_col++;
         file_col++;
         buf_ptr->length = len + 1;
         modified = TRUE;
         }
      }
   else   /* typeover */
      {
      putchar(key);
      modified = TRUE;
      buf_ptr->info[screen_col] = key;
      if (screen_col == 79)
         {
         beep();
         buf_ptr->length = 80;
         }
      else
         {
         screen_col++;
         file_col++;
         if (screen_col > len)
            buf_ptr->length = screen_col;
         }
      }

   show_file_col();
   }


main(int argc, char *argv[])
   {
   INT8 code, file_index, num_index;
   INT32 key, pages;
   FILE *source, *backup;
   char backup_string[80];

   proceed = TRUE;

   if ((argc != 2) && (argc != 3))
      syntax_error();

   if (argc == 2)
      {
      strcpy(source_filename, argv[1]);
      buffer_size = DEFAULT_PAGES * LINES_PER_PAGE;
      }
   else
      {
      if (argv[1][0] == '-')
         {
         file_index = 2;
         num_index = 1;
         }
      else if (argv[2][0] == '-')
         {
         file_index = 1;
         num_index = 2;
         }
      else
         syntax_error();

      pages = -atoi(argv[num_index]);
      if ((pages >= 1) && (pages <= 100))
         {
         buffer_size = pages * LINES_PER_PAGE;
         strcpy(source_filename, argv[file_index]);
         }
      else
         syntax_error();
      }

   if (source_filename[0] == '-')
      {
      fprintf(stderr, "Illegal filename: %s\n", source_filename);
      exit(1);
      }

   buffer = malloc(buffer_size * sizeof(LINE_TYPE));
   if (buffer == NULL)
      {
      fprintf(stderr, "Insufficient memory space\n");
      fprintf(stderr,
         "Did you use the -n option to specify the number of pages?\n");
      exit(1);
      }

   buf_initialize();

   if (display_init() == -1)
      {
      fprintf(stderr, "\nError in initialization of output routines\n");
      exit(1);
      }

   if (getch_init() == -1)
      ERROR("Error in initialization of input routines")

   source = fopen(source_filename, "r");
   if (source == NULL)
      {
      source = fopen(source_filename, "w");
      if (source == NULL)
         {
         getch_done();
         display_color(NORMAL);
         display_clear();
         printf("\nCannot open file %s for reading or writing\n",
            source_filename);
         exit(1);
         }
      fclose(source);
      mode = NEWFILE;
      }
   else
      {
      get_backup_name(source_filename, backup_string);
      backup = fopen(backup_string, "w");
      if (backup == NULL)
         {
         getch_done();
         display_color(NORMAL);
         display_clear();
         printf("\nCannot open file %s for writing\n", backup_string);
         exit(1);
         }
      mode = OLDFILE;
      }

   /* Display top 2 lines of screen before trying to load an old file: */
   insert = TRUE;
   modified = FALSE;
   screen_row = 2;
   screen_col = 0;
   file_row = 1;
   file_col = 1;
   display_color(NORMAL);
   display_info_lines();
   TargetLength = 0;
   mark.num_marks = 0;
   BlockMarked = FALSE;
   clipboard = NIL;

   if (mode == OLDFILE)
      {
      read_file(source);
      fclose(source);
      if (buffer_to_file(backup, first, last) == FALSE)
         ERROR("Error in writing to file")
      fclose(backup);
      }
   else   /* we have a new file */
      {
      if ((first = get_buffer_line()) == NIL)
         ERROR("Unable to allocate memory space")

      current = first;
      last = first;
      buffer[first].left = NIL;
      buffer[first].right = NIL;
      line_count = 1;
      }

   display_goto(screen_row, screen_col);

   while (proceed)
      {
      code = (INT8) getkey(&key);

      if (code == PRINTABLE_KEY)
         {
         if (BlockMarked)
            beep();
         else
            process_printable(key);
         }
      else if (code == SPECIAL_KEY)
         process_special(key);
      else   /* unknown key */
         beep();
      }

   if (getch_done() == -1)
      ERROR("Error in shut down of immediate input")

   display_color(NORMAL);
   display_clear();
   free( (void *) buffer);
   return 0;
   }

